import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * Your character that you can control with the arrow keys and <a> and
 * <d>. Poppi can eat 'good' food and 'bad' food (mushroom).
 * 
 * @author Folkert van Verseveld 
 * @version 0.1.0
 */
public class Poppi  extends Actor
{
    /** the gravity speed of your Poppi */
    private static final float gravity_speed = 0.2F;
    /** the vspeed (VerticalSpeed) of your Poppi */
    private float vspeed = 0F;
    private int x = 0;
    private int y = 0;
    /** 
     * the minimum y that Poppi must have to move faster higher
     * into the air.
     */
    private int minY = 80;
    private int dif = 0;
    private int sadTime = 0; // When Poppi eats some bad food and sadTime == 0
                             // than is Poppi again happy.
    private int mushroomCount = 0; // The total mushrooms that Poppi have eaten.
    private boolean dead = false;
    private boolean healthDead = false; // when poppis health <= 0
    
    // Initialize the images
    GreenfootImage normalImage = getImage();
    GreenfootImage deadImage = new GreenfootImage("smiley3.png");
    GreenfootImage sadImage = deadImage;
    /**
     * Do whatever the Poppi wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act() 
    {
        x = getX();
        y = getY();
        if (!dead && ((PoppiWorld) getWorld()).GameStarted)
        {
            Gravity();
            y += (int)vspeed;
        }
        if (sadTime > 0)
        {
            sadTime--;
            if (sadTime <= 0)
            {
                happy();
            }
        }
        if (((PoppiWorld) getWorld()).GameStarted)
        {
            xWrap();
            setLocation(x, y);
            minY = (getVspeed() < -10) ? 160 : 80;
            if (y < minY)
            {
                dif = (int) (getY() / 8); // dif means difference.
                List<Bar> allBars = getWorld().getObjects(Bar.class);
                for (Bar c : allBars)
                {
                    c.setLocation(c.getX(), c.getY() + dif);
                }
                List<Bar2> allBars2 = getWorld().getObjects(Bar2.class);
                for (Bar2 c : allBars2)
                {
                    c.setLocation(c.getX(), c.getY() + dif);
                }
                List<Food> allFood = getWorld().getObjects(Food.class);
                for (Food c : allFood)
                {
                    c.setLocation(c.getX(), c.getY() + dif);
                }
                setY(getY() + dif);
                ((PoppiWorld) getWorld()).LastBarDistance += dif / 2;
                ((PoppiWorld) getWorld()).TimeFoodOutPoppiWorld += dif / 2;
                ((PoppiWorld) getWorld()).Bar2Time -= dif / 2;
                ((PoppiWorld) getWorld()).counter.add(1);
            }
            if (deadCheck())
            {
                setY(454);
                dead = true;
                setImage(deadImage);
                ((PoppiWorld) getWorld()).gameOver();
            }
            movement();
        }
    }
    
    /**
     * Relative the vspeed with the gravity_speed until if vspeed
     * has reached his limit.
     */
    public void Gravity()
    {
        if (vspeed < 16)
        {
            vspeed += gravity_speed;
        }
    }
    
    /**
     * Returns a boolean if Papi is in the dead zone.
     */
    public boolean deadCheck()
    {
        return (getY() > 440);
    }
    
    /**
     * Sets Poppis happyness to sad and sets the sadTime(r)
     * @param sadTime the time that Poppi is sad in seconds * simulation_speed
     */
    public void sad(int sadTime)
    {
        setImage(sadImage);
        Greenfoot.playSound("vomit.wav");
        this.sadTime = sadTime;
        mushroomCount += 10;
        ((PoppiWorld) getWorld()).healthBar.add(-20);
        if (mushroomCount >= 50)
        {
            healthDead = true;
            this.sadTime = 300;
        }
    }
    
    /**
     * Sets Poppis happyness from sad to happy
     */
    public void happy()
    {
        sadTime = 0;
        if (!dead)
        {
            setImage(normalImage);
        }
    }
    
    /**
     * Sets this.x to x and set the location to this.x.
     * @param x the new x of Poppi
     */
    public void setX(int x)
    {
        this.x = x;
        setLocation(x, getY());
    }
    
    /**
     * Sets this.y to y and set the location to this.y.
     * @param y the new y of Poppi
     */
    public void setY(int y)
    {
        if (y < 4)
        {
            y = 4;
        }
        this.y = y;
        setLocation(getX(), y);
    }
    
    /**
     * Check if the user pressed the left or right arrow or <a> or <d> and
     * Poppi moves to that direction.
     */
    public void movement()
    {
        if (Greenfoot.isKeyDown("left")  || Greenfoot.isKeyDown("a"))
        {
            setX(getX() - 4);
        }
        if (Greenfoot.isKeyDown("right") || Greenfoot.isKeyDown("d"))
        {
            setX(getX() + 4);
        }
    }
    
    /**
     * Let Poppi jump into the air.
     */
    public void jump()
    {
        vspeed = -8;
    }
    
    /**
     * Let Poppi jump high into the air.
     */
    public void highJump()
    {
        vspeed = -20;
    }
    
    /**
     * If the x of Poppi is less than 10 or greater than 630 than must
     * Poppi wrap to the other side of the room.
     */
    public void xWrap()
    {
        if (getX() < 10)
        {
            x = 630;
        }
        if (getX() > 630)
        {
            x = 10;
        }
    }
    
    /**
     * Returns float of vspeed
     */
    public float getVspeed()
    {
        return vspeed;
    }
    
    /**
     * Sets float this.vspeed to float vspeed
     * @param vspeed the new vspeed of Poppi
     */
    public void setVspeed(float vspeed)
    {
        this.vspeed = vspeed;
    }
    
    /**
     * Resets Poppi.
     */
    public void reset()
    {
        setX(320);
        setY(80);
        vspeed = 0;
        dead = false;
        healthDead = false;
        mushroomCount = 0;
        happy();
    }
    
    /**
     * Return a boolean if poppis health <= 0
     */
    public boolean getHealthDead()
    {
        return healthDead;
    }
    
    /**
     * When Poppi eats something that isn't bad for him.
     */
    public void goodEat()
    {
        if (mushroomCount > 0)
        {
            mushroomCount--;
        }
        HealthBar healthBar = ((PoppiWorld) getWorld()).healthBar;
        healthBar.add(2);
    }
}
