import greenfoot.*;  // (World, Actor, GreenfootImage, Greenfoot and MouseInfo)
import java.util.List;

/**
 * PapiWorld - The world where Papi jumps!
 * 
 * @author Folkert van Verseveld 
 * @version 0.1.0
 */
public class PoppiWorld  extends World
{
    Counter counter = new Counter("PoppiJump - Score: ");
    HealthBar healthBar = new HealthBar("Health: ");
    int BarCount;        // the count of bars in the PapiWorld
    int LastBarDistance = 0; // the distance between the last bar.getY() and 0
    boolean FoodInPoppiWorld = true;
    int TimeFoodOutPoppiWorld = 0;
    boolean GameStarted = false;
    int difficult = 0; // how higher Poppi comes, how bigger difficult is.
    int Bar2Time = 1600;
    Poppi poppiObject = new Poppi();
    
    /**
     * Constructor for objects of class PapiWorld.
     * 
     */
    public PoppiWorld()
    {    
        // Create a new world with 640x480 cells with a cell size of 1x1 pixel.
        super(640, 480, 1);
        setPaintOrder(ScoreBoard.class,GameStartBoard.class,Counter.class,HealthBar.class,Poppi.class,Food.class,Bar2.class,Bar.class);
        addObject(poppiObject, 320, 80);
        addObject(new Food(20, true), 320, 240);
        addObject(counter, 200, 440);
        addObject(healthBar, 200, 464);
        addBars();
        addObject(new GameStartBoard(), getWidth() / 2, getHeight() / 2);
        Greenfoot.start();
    }
    
    /**
     * addBars - Add the standard Bars in the room.
     */
    public void addBars()
    {
        BarCount = 6;
        addObject(new Bar(), 320, 320);
        addObject(new Bar(), 192, 128);
        addObject(new Bar(), 448, 48);
        addObject(new Bar(), 64, 256);
        addObject(new Bar(), 352, 464);
        addObject(new Bar(), 224, 80);
    }
    
    /**
     * Act - do whatever the PoppiWorld wants to do. This method is called whenever
     * the 'Act' or 'Run' button gets pressed in the environment.
     */
    public void act()
    {
        if (GameStarted)
        {
            LastBarDistance++;
            Bar2Time--;
            setDifficult();
            if(Greenfoot.getRandomNumber(100) < 2 || BarCount < (8 - difficult) || LastBarDistance > (64 + difficult * 16))
            {
                BarCount++;
                LastBarDistance = 0;
                addObject(new Bar(), 32 + Greenfoot.getRandomNumber(576), 0);
            }
            if (Bar2Time <= 0)
            {
                Bar2Time = 1600;
                addObject(new Bar2(), 32 + Greenfoot.getRandomNumber(576), 0);
            }
            if (!FoodInPoppiWorld)
            {
                TimeFoodOutPoppiWorld++;
                if (TimeFoodOutPoppiWorld > 250 || Greenfoot.getRandomNumber(100) < 2)
                {
                    TimeFoodOutPoppiWorld = 0;
                    FoodInPoppiWorld = true;
                    switch((int)Greenfoot.getRandomNumber(6))
                    {
                        case 0:
                            addObject(new Bread(), 32 + Greenfoot.getRandomNumber(576), 0);
                            break;
                        case 1:
                            addObject(new Food(20, true), 32 + Greenfoot.getRandomNumber(576), 0);
                            break;
                        case 2:
                            addObject(new Chips(), 32 + Greenfoot.getRandomNumber(576), 0);
                            break;
                        case 3:
                            addObject(new Pizza(), 32 + Greenfoot.getRandomNumber(576), 0);
                            break;
                        case 4:
                            addObject(new Mushroom(), 32 + Greenfoot.getRandomNumber(576), 0);
                            break;
                        case 5:
                            addObject(new Muffin(), 32 + Greenfoot.getRandomNumber(576), 0);
                            break;
                    }
                }
            }
        }
    }
    
    /**
     * setDifficult - Sets the game difficult
     */
    public void setDifficult()
    {
        if (counter.getTarget() > 800)
        {
            difficult = 4;
        }
        else
        if (counter.getTarget() > 500)
        {
            difficult = 3;
        }
        else if (counter.getTarget() > 300)
        {
            difficult = 2;
        }
        else if (counter.getTarget() > 180)
        {
            difficult = 1;
        }
    }
    
    /**
     * Count one eated food.
     */
    public void countFood(int count)
    {
        counter.add(count);
    }
    
    /**
     * Called when game is up. Stop running and display score.
     */
    public void gameOver() 
    {
        ScoreBoard scoreBoard = new ScoreBoard(counter.getTarget());
        addObject(scoreBoard, getWidth() / 2, getHeight() / 2);
        Greenfoot.playSound("buzz.wav");
        if (counter.getTarget() < 200)
        {
            Greenfoot.delay(45);
            Greenfoot.playSound("laugh.wav");
        }
        else if (counter.getTarget() >= 800)
        {
            Greenfoot.delay(45);
            Greenfoot.playSound("good.wav");
        }
        GameStarted = false;
    }
    
    /**
     * restart - Restarts the PoppiWorld.
     */
    public void restartGame()
    {
        difficult = 0;
        poppiObject.reset();
        poppiObject.setImage("smiley2.png");
        List<Food> allFeed = getObjects(Food.class);
        for (Food c : allFeed)
        {
            removeObject(c);
        }
        addObject(new Food(20, true), 320, 240);
        List<Bar> allBars = getObjects(Bar.class);
        for (Bar c : allBars)
        {
            removeObject(c);
        }
        List<Bar2> allBars2 = getObjects(Bar2.class);
        for (Bar2 c : allBars2)
        {
            removeObject(c);
        }
        addBars();
        counter.reset();
        healthBar.reset();
        addObject(new GameStartBoard(), getWidth() / 2, getHeight() / 2);
    }
}
